import type { TransactionResponse } from "./types"
import * as querystring from "node:querystring"
import { toISODateString } from "@common/date"
import logger from "@common/logger"
import { baseUrl } from "./common"
import type { Interval } from "@common/types.ts"

export async function list(
  accessToken: string,
  accountKeys: string | ReadonlyArray<string>,
  interval?: Interval,
): Promise<TransactionResponse> {
  const queryString = querystring.stringify({
    accountKey: accountKeys,
    ...(interval && {
      fromDate: toISODateString(interval.fromDate),
      toDate: toISODateString(interval.toDate),
    }),
  })

  const url = `${baseUrl}/personal/banking/transactions?${queryString}`
  logger.info(`GET '${url}'`)
  const response = await fetch(url, {
    headers: {
      Authorization: `Bearer ${accessToken}`,
      Accept: "application/vnd.sparebank1.v1+json;charset=utf-8",
    },
  })
  logger.debug(`Received response with status '${response.status}'`)
  if (response.ok) {
    return response.json()
  } else {
    logger.warn(await response.json())
    return { transactions: [] }
  }
}
